from .hopper_controller_oracle import HopperControllerOracle
from .ant_morphology_oracle import AntMorphologyOracle
from .dkitty_morphology_oracle import DKittyMorphologyOracle
from .toy_continuous_oracle import ToyContinuousOracle
from .nas_bench_oracle import NASBenchOracle
from .tf_bind_8_oracle import TFBind8Oracle
from .tf_bind_10_oracle import TFBind10Oracle
from .toy_discrete_oracle import ToyDiscreteOracle
